/*
 Copyright 2009-2011 Urban Airship Inc. All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binaryform must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided withthe distribution.

 THIS SOFTWARE IS PROVIDED BY THE URBAN AIRSHIP INC``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 EVENT SHALL URBAN AIRSHIP INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "PushPlugin.h"

@interface PushPlugin ()<UNUserNotificationCenterDelegate>

@end

@implementation PushPlugin

@synthesize notificationMessage;
@synthesize isInline;

@synthesize callbackId;
@synthesize notificationCallbackId;
@synthesize callback;

- (void)pluginInitialize{
    if (IsAtLeastiOSVersion(@"10.0")) {
      center = [UNUserNotificationCenter currentNotificationCenter];
      center.delegate = self;
    }
}

- (void)unregister:(CDVInvokedUrlCommand*)command;
{
  self.callbackId = command.callbackId;
  [[UIApplication sharedApplication] unregisterForRemoteNotifications];
  [self successWithMessage:@"unregistered"];
}

- (void)areNotificationsEnabled:(CDVInvokedUrlCommand*)command;
{
  self.callbackId = command.callbackId;
  BOOL registered = false;
  if ([[UIApplication sharedApplication] respondsToSelector:@selector(isRegisteredForRemoteNotifications)]) {
    registered = [[UIApplication sharedApplication] isRegisteredForRemoteNotifications];
  }else {
    UIRemoteNotificationType types = [[UIApplication sharedApplication] enabledRemoteNotificationTypes];
    registered = types != UIRemoteNotificationTypeNone;
  }
  
  CDVPluginResult *commandResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsBool:registered];
  [self.commandDelegate sendPluginResult:commandResult callbackId:self.callbackId];
}

- (void)registerUserNotificationSettings:(CDVInvokedUrlCommand*)command;
{
  self.callbackId = command.callbackId;
  
  NSDictionary *options = [command.arguments objectAtIndex:0];
  NSArray *types = [options objectForKey:@"types"];
  if (types == nil) {
    [self failWithMessage:@"No types specified" withError:nil];
    return;
  }
  
  if (IsAtLeastiOSVersion(@"10.0")) {
    UNAuthorizationOptions nsTypes = 0;
    for (NSString *type in types) {
      if ([type isEqualToString:@"badge"]) {
        nsTypes |= UNAuthorizationOptionBadge;
      } else if ([type isEqualToString:@"alert"]) {
        nsTypes |= UNAuthorizationOptionAlert;
      } else if ([type isEqualToString:@"sound"]) {
        nsTypes |= UNAuthorizationOptionSound;
      } else {
        [self failWithMessage:[NSString stringWithFormat:@"Unsupported type: %@, use one of badge, alert, sound", type] withError:nil];
      }
    }
    
    [center requestAuthorizationWithOptions:nsTypes completionHandler:^(BOOL granted, NSError * _Nullable error){
      if( !error ){
        [[UIApplication sharedApplication] registerForRemoteNotifications];
      }
    }];
  } else if (IsAtLeastiOSVersion(@"8.0")) {
    if (![[UIApplication sharedApplication] respondsToSelector:@selector(registerUserNotificationSettings:)]) {
      [self successWithMessage:[NSString stringWithFormat:@"%@", @"user notifications not supported for this ios version."]];
      return;
    }
  }
  
  NSArray *categories = [options objectForKey:@"categories"];
  if (categories == nil) {
    [self failWithMessage:@"No categories specified" withError:nil];
    return;
  }
  
  NSMutableArray *nsCategories = [[NSMutableArray alloc] initWithCapacity:[categories count]];
  for (NSDictionary *category in categories) {
    // ** 1. create the actions for this category
    NSMutableArray *nsActionsForDefaultContext = [[NSMutableArray alloc] initWithCapacity:4];
    NSArray *actionsForDefaultContext = [category objectForKey:@"actionsForDefaultContext"];
    if (actionsForDefaultContext == nil) {
      [self failWithMessage:@"Category doesn't contain actionsForDefaultContext" withError:nil];
      return;
    }
    
    if (![self createNotificationAction:category actions:actionsForDefaultContext nsActions:nsActionsForDefaultContext]) {
      return;
    }
    
    NSMutableArray *nsActionsForMinimalContext = [[NSMutableArray alloc] initWithCapacity:2];
    NSArray *actionsForMinimalContext = [category objectForKey:@"actionsForMinimalContext"];
    if (actionsForMinimalContext == nil) {
      [self failWithMessage:@"Category doesn't contain actionsForMinimalContext" withError:nil];
      return;
    }
    
    if (![self createNotificationAction:category actions:actionsForMinimalContext nsActions:nsActionsForMinimalContext]) {
      return;
    }
    
    // ** 2. create the category
    UIMutableUserNotificationCategory *nsCategory = [[UIMutableUserNotificationCategory alloc] init];
    // Identifier to include in your push payload and local notification
    NSString *identifier = [category objectForKey:@"identifier"];
    if (identifier == nil) {
      [self failWithMessage:@"Category doesn't contain identifier" withError:nil];
      return;
    }
    
    nsCategory.identifier = identifier;
    // Add the actions to the category and set the action context
    [nsCategory setActions:nsActionsForDefaultContext forContext:UIUserNotificationActionContextDefault];
    // Set the actions to present in a minimal context
    [nsCategory setActions:nsActionsForMinimalContext forContext:UIUserNotificationActionContextMinimal];
    [nsCategories addObject:nsCategory];
  }
  
  UIUserNotificationType nsTypes = UIUserNotificationTypeNone;
  for (NSString *type in types) {
    if ([type isEqualToString:@"badge"]) {
      nsTypes |= UIUserNotificationTypeBadge;
    } else if ([type isEqualToString:@"alert"]) {
      nsTypes |= UIUserNotificationTypeAlert;
    } else if ([type isEqualToString:@"sound"]) {
      nsTypes |= UIUserNotificationTypeSound;
    } else {
      [self failWithMessage:[NSString stringWithFormat:@"Unsupported type: %@, use one of badge, alert, sound", type] withError:nil];
    }
  }
  
  // ** 4. Register the notification categories
  NSSet *nsCategorySet = [NSSet setWithArray:nsCategories];
  UIUserNotificationSettings *settings = [UIUserNotificationSettings settingsForTypes:nsTypes categories:nsCategorySet];
  [[UIApplication sharedApplication] registerUserNotificationSettings:settings];
  [[UIApplication sharedApplication] registerForRemoteNotifications];
  
  [self successWithMessage:[NSString stringWithFormat:@"%@", @"user notifications registered"]];
}

- (BOOL)createNotificationAction:(NSDictionary *)category
                         actions:(NSArray *) actions
                       nsActions:(NSMutableArray *)nsActions
{
  for (NSDictionary *action in actions) {
    UIMutableUserNotificationAction *nsAction = [[UIMutableUserNotificationAction alloc] init];
    // Define an ID string to be passed back to your app when you handle the action
    NSString *identifier = [action objectForKey:@"identifier"];
    if (identifier == nil) {
      [self failWithMessage:@"Action doesn't contain identifier" withError:nil];
      return NO;
    }
    nsAction.identifier = identifier;
    // Localized text displayed in the action button
    NSString *title = [action objectForKey:@"title"];
    if (title == nil) {
      [self failWithMessage:@"Action doesn't contain title" withError:nil];
      return NO;
    }
    nsAction.title = title;
    // If you need to show UI, choose foreground (background gives your app a few seconds to run)
    BOOL isForeground = [@"foreground" isEqualToString:[action objectForKey:@"activationMode"]];
    nsAction.activationMode = isForeground ? UIUserNotificationActivationModeForeground : UIUserNotificationActivationModeBackground;
    // Destructive actions display in red
    BOOL isDestructive = [[action objectForKey:@"destructive"] isEqual:[NSNumber numberWithBool:YES]];
    nsAction.destructive = isDestructive;
    // Set whether the action requires the user to authenticate
    BOOL isAuthRequired = [[action objectForKey:@"authenticationRequired"] isEqual:[NSNumber numberWithBool:YES]];
    nsAction.authenticationRequired = isAuthRequired;
    [nsActions addObject:nsAction];
  }
  return YES;
}

- (void)register:(CDVInvokedUrlCommand*)command;
{
  self.callbackId = command.callbackId;
  
  NSMutableDictionary* options = [command.arguments objectAtIndex:0];
  
  UNAuthorizationOptions nsTypes = 0;
  UIUserNotificationType UserNotificationTypes = UIUserNotificationTypeNone;
  UIRemoteNotificationType notificationTypes = UIRemoteNotificationTypeNone;
  
  id badgeArg = [options objectForKey:@"badge"];
  id soundArg = [options objectForKey:@"sound"];
  id alertArg = [options objectForKey:@"alert"];
  
  if ([badgeArg isKindOfClass:[NSString class]]){
    if ([badgeArg isEqualToString:@"true"]) {
      notificationTypes |= UIRemoteNotificationTypeBadge;
      UserNotificationTypes |= UIUserNotificationTypeBadge;
      nsTypes |= UNAuthorizationOptionBadge;
    }
  }else if ([badgeArg boolValue]) {
    notificationTypes |= UIRemoteNotificationTypeBadge;
    UserNotificationTypes |= UIUserNotificationTypeBadge;
    nsTypes |= UNAuthorizationOptionBadge;
  }
  
  if ([soundArg isKindOfClass:[NSString class]])
  {
    if ([soundArg isEqualToString:@"true"]) {
      notificationTypes |= UIRemoteNotificationTypeSound;
      UserNotificationTypes |= UIUserNotificationTypeSound;
      nsTypes |= UNAuthorizationOptionSound;
    }
  }else if ([soundArg boolValue]) {
    notificationTypes |= UIRemoteNotificationTypeSound;
    UserNotificationTypes |= UIUserNotificationTypeSound;
    nsTypes |= UNAuthorizationOptionSound;
  }
  
  if ([alertArg isKindOfClass:[NSString class]]){
    if ([alertArg isEqualToString:@"true"]) {
      notificationTypes |= UIRemoteNotificationTypeAlert;
      UserNotificationTypes |= UIUserNotificationTypeAlert;
      nsTypes |= UNAuthorizationOptionAlert;
    }
  }else if ([alertArg boolValue]) {
    notificationTypes |= UIRemoteNotificationTypeAlert;
    UserNotificationTypes |= UIUserNotificationTypeAlert;
    nsTypes |= UNAuthorizationOptionAlert;
  }
  
  self.callback = [options objectForKey:@"onMessage"];
  isInline = NO;
  
  if (IsAtLeastiOSVersion(@"10.0")) {
    [center requestAuthorizationWithOptions:nsTypes completionHandler:^(BOOL granted, NSError * _Nullable error){
      if (granted) {
        NSLog(@"请求成功");
        [[UIApplication sharedApplication] registerForRemoteNotifications];
      } else {
        NSLog(@"请求失败");
      }
    }];
  }else if (IsAtLeastiOSVersion(@"8.0")) {
    if ([[UIApplication sharedApplication]respondsToSelector:@selector(registerUserNotificationSettings:)]) {
      UIUserNotificationSettings *settings = [UIUserNotificationSettings settingsForTypes:UserNotificationTypes categories:nil];
      [[UIApplication sharedApplication] registerUserNotificationSettings:settings];
      [[UIApplication sharedApplication] registerForRemoteNotifications];
    }
  }else{
    [[UIApplication sharedApplication] registerForRemoteNotificationTypes:notificationTypes];
  }
  
  // if there is a pending startup notification
  if (notificationMessage){
    [self notificationReceived];  // go ahead and process it
  }
}

- (void)didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
  
  NSMutableDictionary *results = [NSMutableDictionary dictionary];
  NSString *token = [[[[deviceToken description] stringByReplacingOccurrencesOfString:@"<"withString:@""]
                      stringByReplacingOccurrencesOfString:@">" withString:@""]
                     stringByReplacingOccurrencesOfString: @" " withString: @""];
  [results setValue:token forKey:@"deviceToken"];
  
  // Get Bundle Info for Remote Registration (handy if you have more than one app)
  [results setValue:[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleDisplayName"] forKey:@"appName"];
  [results setValue:[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"] forKey:@"appVersion"];
  
  // Check what Notifications the user has turned on.  We registered for all three, but they may have manually disabled some or all of them.
  NSUInteger rntypes;
  
  // Set the defaults to disabled unless we find otherwise...
  NSString *pushBadge = @"disabled";
  NSString *pushAlert = @"disabled";
  NSString *pushSound = @"disabled";
  
  if (IsAtLeastiOSVersion(@"8.0")) { //iOS8以上包含iOS8
    rntypes = [[UIApplication sharedApplication] currentUserNotificationSettings].types;
    if(rntypes & UIUserNotificationTypeBadge){
      pushBadge = @"enabled";
    }
    if(rntypes & UIUserNotificationTypeAlert) {
      pushAlert = @"enabled";
    }
    if(rntypes & UIUserNotificationTypeSound) {
      pushSound = @"enabled";
    }
  }else{ // ios7 一下
    rntypes = [[UIApplication sharedApplication] enabledRemoteNotificationTypes];
    if(rntypes & UIRemoteNotificationTypeBadge){
      pushBadge = @"enabled";
    }
    if(rntypes & UIRemoteNotificationTypeAlert) {
      pushAlert = @"enabled";
    }
    if(rntypes & UIRemoteNotificationTypeSound) {
      pushSound = @"enabled";
    }
  }
  
  [results setValue:pushBadge forKey:@"pushBadge"];
  [results setValue:pushAlert forKey:@"pushAlert"];
  [results setValue:pushSound forKey:@"pushSound"];
  
  // Get the users Device Model, Display Name, Token & Version Number
  UIDevice *dev = [UIDevice currentDevice];
  [results setValue:dev.name forKey:@"deviceName"];
  [results setValue:dev.model forKey:@"deviceModel"];
  [results setValue:dev.systemVersion forKey:@"deviceSystemVersion"];
  
  NSLog(@"token = %@", [NSString stringWithFormat:@"%@", token]);
  [self successWithMessage:[NSString stringWithFormat:@"%@", token]];
}

- (void)didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
  [self failWithMessage:@"" withError:error];
}

- (void)notificationReceived {
  NSLog(@"Notification received");
  
  if (notificationMessage && self.callback)
  {
    NSMutableString *jsonStr = [NSMutableString stringWithString:@"{"];
    
    [self parseDictionary:notificationMessage intoJSON:jsonStr];
    
    if (isInline)
    {
      [jsonStr appendFormat:@"foreground:\"%d\"", 1];
      isInline = NO;
    }
    else
      [jsonStr appendFormat:@"foreground:\"%d\"", 0];
    
    [jsonStr appendString:@"}"];
    
    NSLog(@"Msg: %@", jsonStr);
    
    NSString * jsCallBack = [NSString stringWithFormat:@"%@(%@);", self.callback, jsonStr];
    [self.webViewEngine evaluateJavaScript:jsCallBack completionHandler:nil];
    self.notificationMessage = nil;
  }
}

// reentrant method to drill down and surface all sub-dictionaries' key/value pairs into the top level json
-(void)parseDictionary:(NSDictionary *)inDictionary intoJSON:(NSMutableString *)jsonString
{
  NSArray         *keys = [inDictionary allKeys];
  NSString        *key;
  
  for (key in keys)
  {
    id thisObject = [inDictionary objectForKey:key];
    
    if ([thisObject isKindOfClass:[NSDictionary class]])
      [self parseDictionary:thisObject intoJSON:jsonString];
    else if ([thisObject isKindOfClass:[NSString class]])
      [jsonString appendFormat:@"\"%@\":\"%@\",",
       key,
       [[[[inDictionary objectForKey:key]
          stringByReplacingOccurrencesOfString:@"\\" withString:@"\\\\"]
         stringByReplacingOccurrencesOfString:@"\"" withString:@"\\\""]
        stringByReplacingOccurrencesOfString:@"\n" withString:@"\\n"]];
    else {
      [jsonString appendFormat:@"\"%@\":\"%@\",", key, [inDictionary objectForKey:key]];
    }
  }
}

- (void)setApplicationIconBadgeNumber:(CDVInvokedUrlCommand *)command {
  
  self.callbackId = command.callbackId;
  
  NSMutableDictionary* options = [command.arguments objectAtIndex:0];
  int badge = [[options objectForKey:@"badge"] intValue] ?: 0;
  
  [[UIApplication sharedApplication] setApplicationIconBadgeNumber:badge];
  
  [self successWithMessage:[NSString stringWithFormat:@"app badge count set to %d", badge]];
}

-(void)successWithMessage:(NSString *)message
{
  if (self.callbackId != nil)
  {
    CDVPluginResult *commandResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:message];
    [self.commandDelegate sendPluginResult:commandResult callbackId:self.callbackId];
  }
}

-(void)failWithMessage:(NSString *)message withError:(NSError *)error
{
  NSString        *errorMessage = (error) ? [NSString stringWithFormat:@"%@ - %@", message, [error localizedDescription]] : message;
  CDVPluginResult *commandResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:errorMessage];
  
  [self.commandDelegate sendPluginResult:commandResult callbackId:self.callbackId];
}

//MARK: UNUserNotificationCenterDelegate
-(void)userNotificationCenter:(UNUserNotificationCenter *)center willPresentNotification:(UNNotification *)notification withCompletionHandler:(void (^)(UNNotificationPresentationOptions options))completionHandler{
  //需要执行这个方法，选择是否提醒用户，有Badge、Sound、Alert三种类型可以设置
  NSLog(@"didReceiveRemoteNotification: %@", notification.request.content.userInfo);
  completionHandler(UNNotificationPresentationOptionBadge|UNNotificationPresentationOptionSound|UNNotificationPresentationOptionAlert);
  //Called when a notification is delivered to a foreground app.
  UIApplication *app = [UIApplication sharedApplication];
  if (app.applicationState == UIApplicationStateActive) {
    self.notificationMessage = notification.request.content.userInfo;
    self.isInline = YES;
    [self notificationReceived];
  } else {
    self.notificationMessage = notification.request.content.userInfo;
    [self performSelectorOnMainThread:@selector(notificationReceived) withObject:self waitUntilDone:NO];
  }
}

-(void)userNotificationCenter:(UNUserNotificationCenter *)center didReceiveNotificationResponse:(UNNotificationResponse *)response withCompletionHandler:(void(^)(void))completionHandler{
  NSLog( @"Handle push from background or closed" );
  completionHandler();
}

@end
